//PICの定義ファイルをインクルード
#include <pic/pic16f88.h>
#include <sdcc-lib.h>

//PICの書き込み設定
int at 0x2007 __config = _INTRC_IO & _WDT_OFF & _PWRTE_OFF & _CP_OFF & _MCLR_OFF & _LVP_OFF & _BODEN_OFF;

/*
ラミーキューブタイマー
RA0,RA1,RB0-RB7 7セグ
RA2 ボリューム
RA3 スピーカー
RA4 ボタン
*/

void wait(long);
unsigned char num2hex(char);
void outnum(char);
void tmr0init(void);
void tmr0start(void);
void tmr0stop(void);


#define DEFTIME	60			// ラミィキューブタイマー初期値

char g_7segcnt;				// ７セグ表示用カウンタ
long g_cdtmr;				// カウントダウンタイマ用カウンタ


// 割り込み処理
static void timer() interrupt 0 {
	TMR0IF = 0;			// 割り込みフラグクリア
	/*** 割り込み時の処理 ***/
	g_cdtmr++;
	outnum(g_7segcnt);
}

// ウエイト
void wait( volatile long time )
{
	while(time > 0){ time--; }
}

// 数値を７セグポート値に変換‘
unsigned char num2hex(char num)
{
	int ret;
	switch(num)
	{
		case 0:
			ret = 0x88;	//10001000
			break;
		case 1:
			ret = 0xBE;	//10111110
			break;
		case 2:
			ret = 0xC4;	//11000100
			break;
		case 3:
			ret = 0x94;	//10010100
			break;
		case 4:
			ret = 0xB2;	//10110010
			break;
		case 5:
			ret = 0x91;	//10010001
			break;
		case 6:
			ret = 0x81;	//10000001
			break;
		case 7:
			ret = 0xBC;	//10111100
			break;
		case 8:
			ret = 0x80;	//10000000
			break;
		case 9:
			ret = 0x90;	//10010000
			break;
		default:
			ret = 0x00;
			break;
	}
	return ret;
}

// 数値を７セグ表示
void outnum(char num)
{
	char num1;
	char num2;
	num1 = num;
	num2 = num;

	// 2桁目
	num2 = num2 / 10;
	PORTB = num2hex(num2);
	PORTA = 0x02;
	wait(100);

	// 1桁目
	if( num1 > 9 ){	num1 = num1 % 10; }
	PORTB = num2hex(num1);
	PORTA = 0x01;
	wait(100);
}


// タイマー０の初期化
void tmr0init(void)
{
	OPTION_REG = 0x04;	// 分周比32
	PSA = 0;		// TM0はプリスケーラ使用
}

// タイマー０開始
void tmr0start(void)
{
	TMR0IF = 0;		// 割り込みフラグクリア
	TMR0IE = 1;		// TMR0オーバーフロー割り込み発生許可
	GIE = 1;		// 割り込み許可
}

// タイマー０停止
void tmr0stop(void)
{
	TMR0IE = 0;		// TMR0オーバーフロー割り込み発生end
	GIE = 0;		// 割り込みend
}


// デバイスの初期設定
void init(void)
{
	OSCCON = 0x70;		//PIC16F88を8MHzの内部クロックで動作させる。
	ANSEL = 0x00;		//全部デジタル入力

	TRISA = 0x00;		//Aポートの設定 (1にすると入力)
	TRISB = 0x00;		//Bポートの設定 
	PORTA = 0x00;
	PORTB = 0xFF;
}


void main(void)
{
	init();					// デバイスの初期化
	tmr0init();

	tmr0start();				// TMR0開始

	// スタート
	while( 1 )
	{
		// ７セグ表示用タイマー
		if(g_cdtmr>=350)		// だいたい一秒（かなり適当）
		{
			g_cdtmr=0;
			g_7segcnt--;
		}

		// ０になったら初期値にもどす
		if(g_7segcnt <= 0){ g_7segcnt=DEFTIME; }
	}
}
